
/* simple test functions for time */

#include <stdio.h>
#include "wine/test.h"
#include "winbase.h"
#include "winerror.h"

#define NEWYEAR_1980_HI 0x01a8e79f
#define NEWYEAR_1980_LO 0xe1d58000

#define MAYDAY_2002_HI 0x01c1f107
#define MAYDAY_2002_LO 0xb82b6000

#define ATIME_HI  0x1c2349b
#define ATIME_LOW 0x580716b0

#define LOCAL_ATIME_HI  0x01c23471
#define LOCAL_ATIME_LOW 0x6f310eb0

#define DOS_DATE(y,m,d) ( (((y)-1980)<<9) | ((m)<<5) | (d) )
#define DOS_TIME(h,m,s) ( ((h)<<11) | ((m)<<5) | ((s)>>1) )


#define SETUP_1980(st) \
    (st).wYear = 1980; \
    (st).wMonth = 1; \
    (st).wDay = 1; \
    (st).wHour = 0; \
    (st).wMinute = 0; \
    (st).wSecond = 0; \
    (st).wMilliseconds = 0;

#define SETUP_2002(st) \
    (st).wYear = 2002; \
    (st).wMonth = 5; \
    (st).wDay = 1; \
    (st).wHour = 12; \
    (st).wMinute = 0; \
    (st).wSecond = 0; \
    (st).wMilliseconds = 0;

#define SETUP_ATIME(st) \
    (st).wYear = 2002; \
    (st).wMonth = 7; \
    (st).wDay = 26; \
    (st).wHour = 11; \
    (st).wMinute = 55; \
    (st).wSecond = 32; \
    (st).wMilliseconds = 123;


static void test_conversions()
{
    FILETIME ft;
    SYSTEMTIME st;

    memset(&ft,0,sizeof ft);

    SETUP_ATIME(st)
    ok (SystemTimeToFileTime(&st,&ft), "Conversion Failed ATIME\n");
    ok( (!((ft.dwHighDateTime != ATIME_HI) || (ft.dwLowDateTime!=ATIME_LOW))),
        "Wrong time for ATIME: %08lx %08lx (correct %08x %08x)\n",
        ft.dwLowDateTime, ft.dwHighDateTime, ATIME_LOW, ATIME_HI);


    SETUP_2002(st)
    ok (SystemTimeToFileTime(&st, &ft), "Conversion failed 2002\n");

    ok( (!((ft.dwHighDateTime != MAYDAY_2002_HI) ||
         (ft.dwLowDateTime!=MAYDAY_2002_LO))),
        "Wrong time for 2002 %08lx %08lx (correct %08x %08x)\n", ft.dwLowDateTime,
        ft.dwHighDateTime, MAYDAY_2002_LO, MAYDAY_2002_HI);


    SETUP_1980(st)
    ok((SystemTimeToFileTime(&st, &ft)), "Conversion failed 1980\n");

    ok( (!((ft.dwHighDateTime!=NEWYEAR_1980_HI) ||
        (ft.dwLowDateTime!=NEWYEAR_1980_LO))) ,
        "Wrong time for 1980 %08lx %08lx (correct %08x %08x)\n", ft.dwLowDateTime,
         ft.dwHighDateTime, NEWYEAR_1980_LO,NEWYEAR_1980_HI  );

    ok(DosDateTimeToFileTime(DOS_DATE(1980,1,1),DOS_TIME(0,0,0),&ft),
        "DosDateTimeToFileTime() failed\n");

    ok( (!((ft.dwHighDateTime!=NEWYEAR_1980_HI) ||
         (ft.dwLowDateTime!=NEWYEAR_1980_LO))),
        "Wrong time DosDateTimeToFileTime %08lx %08lx (correct %08x %08x)\n",
        ft.dwHighDateTime, ft.dwLowDateTime, NEWYEAR_1980_HI, NEWYEAR_1980_LO);

}

static void test_timezones()
{
    FILETIME ft,lft;
    SYSTEMTIME st,lst,lstt;
    TIME_ZONE_INFORMATION tzi;

    memset(&ft,0,sizeof ft);
    memset(&lft,0xff,sizeof lft);

    /* FIXME: getting the time this way creates a race condition
        that causes the test to pass sometimes and fail others */
    GetSystemTime(&st);
    GetLocalTime(&lst);

    ok ( ((SystemTimeToFileTime(&st,&ft))
        && (SystemTimeToFileTime(&lst,&lft))
        && (LocalFileTimeToFileTime(&lft,&lft)) ),
        "LocalFileTimeToFileTime failed\n");

    ok ( (!((ft.dwHighDateTime != lft.dwHighDateTime) || (ft.dwLowDateTime !=
          lft.dwLowDateTime))),
        "Wrong time LocalFileTimeToFileTime %08lx %08lx vs %08lx %08lx\n",
        lft.dwLowDateTime, lft.dwHighDateTime,  ft.dwLowDateTime,
        ft.dwHighDateTime );

    memset(&ft,0,sizeof ft);
    memset(&lft,0xff,sizeof lft);
    GetSystemTime(&st);
    GetLocalTime(&lst);

    st.wMilliseconds = 0;
    lst.wMilliseconds = 0;

    ok ((SystemTimeToFileTime(&st,&ft)
        && SystemTimeToFileTime(&lst,&lft)
        && FileTimeToLocalFileTime(&ft,&ft)),"FileTimeToLocalFileTime Failed\n");

    ok ( (!((ft.dwHighDateTime != lft.dwHighDateTime) || (ft.dwLowDateTime !=
           lft.dwLowDateTime))),
        "Wrong time FileTimeToLocalFileTime %08lx %08lx vs %08lx %08lx\n",
        lft.dwLowDateTime, lft.dwHighDateTime,  ft.dwLowDateTime,
        ft.dwHighDateTime );


    /*
     * SystemTimeToTzSpecificLocalTime is stubbed right now
     */

    memset(&ft,0,sizeof ft);
    memset(&lft,0xff,sizeof lft);
    GetSystemTime(&st);
    GetLocalTime(&lst);

    ok ((GetTimeZoneInformation(&tzi)
        && SystemTimeToTzSpecificLocalTime(&tzi,&st,&lstt)
        && SystemTimeToFileTime(&lst,&ft)
        && SystemTimeToFileTime(&lstt,&lft)),
        "SystemTimeToTzSpecificLocalTime Failed\n");

    ok ( (!((ft.dwHighDateTime != lft.dwHighDateTime) || (ft.dwLowDateTime !=
          lft.dwLowDateTime))),
        "Wrong time SystemTimeToTzSpecificLocalTime %08lx %08lx vs %08lx %08lx\n",
        lft.dwLowDateTime, lft.dwHighDateTime, ft.dwLowDateTime,
        ft.dwHighDateTime );


    memset(&ft,0,sizeof ft);
    memset(&lft,0xff,sizeof lft);
    SETUP_ATIME(st)
    SystemTimeToFileTime(&st,&ft);

    ok  ((FileTimeToLocalFileTime(&ft,&lft)),
            "FileTimeToLocalFileTime failed\n");

#if 0
    printf("Only succeeds in Central Daylight Time\n");
    ok ( (!((lft.dwHighDateTime != LOCAL_ATIME_HI) ||
        (lft.dwLowDateTime!=LOCAL_ATIME_LOW) )),
        "Wrong time SystemTimeToFileTime %08lx %08lx (correct %08x %08x)\n",
        lft.dwLowDateTime, lft.dwHighDateTime, LOCAL_ATIME_HI,
        LOCAL_ATIME_LOW);
#endif
}

static void test_invalid_arg()
{
    FILETIME ft;
    SYSTEMTIME st;

    memset(&ft,0,sizeof ft);

    /* Invalid argument checks */

    if(DosDateTimeToFileTime(0,0,&ft)) /* invalid dos date/time */
        printf("DosDateTimeToFileTime() didn't fail\n");

    if(!DosDateTimeToFileTime(DOS_DATE(1980,1,1),DOS_TIME(0,0,0),&ft)) /* this is 1 Jan 1980 00:00:00 */
        printf("DosDateTimeToFileTime() failed\n");

    if( (ft.dwHighDateTime!=NEWYEAR_1980_HI) || (ft.dwLowDateTime!=NEWYEAR_1980_LO) )
        printf("filetime for 1/1/80 00:00:00 was %08lx %08lx\n", ft.dwHighDateTime, ft.dwLowDateTime);

    /* now check SystemTimeToFileTime */
    memset(&ft,0,sizeof ft);


    /* try with a bad month */
    SETUP_1980(st)
    st.wMonth = 0;

    if(SystemTimeToFileTime(&st, &ft))
        printf("bad month\n");

    /* with a bad day */
    SETUP_1980(st)
    st.wDay = 0;

    if(SystemTimeToFileTime(&st, &ft))
        printf("bad day\n");

    /* with a bad hour */
    SETUP_1980(st)
    st.wHour = 25;

    if(SystemTimeToFileTime(&st, &ft))
        printf("bad hour\n");

    /* with a bad minute */
    SETUP_1980(st)
    st.wMinute = 61;

    if(SystemTimeToFileTime(&st, &ft))
        printf("bad minute\n");
}

START_TEST(time)
{
    test_conversions();
    /* test_timezones(); */
    /* test_invalid_arg(); */
}
